import 'package:easy/src/easy_bloc.dart';
import 'package:flutter/material.dart';
import 'inject.dart';
import 'bloc.dart';
import 'core.dart';

final Map<String, Core> _injectMap = {};
final Map<String, Core> _injectMapHelper = {};

class EasyManager extends StatefulWidget {
  EasyManager({
    Key key,
    @required this.view,
    this.bloc,
    this.blocs,
    this.tagText = "global",
  }) : super(key: key);

  final List<Bloc> blocs;
  final String tagText;
  // final List<Widget> views;
  final Widget view;

  final EasyBloc bloc;

  @override
  _EasyManagerListState createState() => _EasyManagerListState();

  static bool debugMode = true;

  static bool isTest = false;

  ///Use to inject a BLoC. If BLoC is not instantiated, it starts a new singleton instance.
  static T getBloc<T extends EasyBloc>(
      [Map<String, dynamic> params, String tag = "global"]) {
    try {
      Core core = _injectMapHelper.containsKey(tag)
          ? _injectMapHelper[tag]
          : _injectMap[tag];
      if (core == null) {
        throw EasyManagerException(
            "Module \"$tag\" is not in the widget tree");
      }
      return core.bloc<T>(params);
    } on EasyManagerException {
      rethrow;
    } on NoSuchMethodError {
      rethrow;
    } catch (e) {
      if (e?.message == "No element") {
        throw EasyManagerException(
            "${T.toString()} is not part of '$tag'. Check Injected BLoC's");
      } else {
        throw e;
      }
    }
  }

  ///tag inject of BLocs, Dependency and Views.
  static Inject tag(String tagText) => Inject(tag: tagText);

  ///Discards BLoC from application memory
  static void disposeBloc<T extends EasyBloc>([String tagText = "global"]) {
    if (_injectMap.containsKey(tagText)) {
      Core core = _injectMap[tagText];
      core?.removeBloc<T>();
    }
  }

  static void addCoreInit(
      List<Bloc> blocs, String tagText) {
    if (!_injectMap.containsKey(tagText)) {
      if (EasyManager.debugMode) {
        print("EasyManager START ($tagText)");
      }
      _injectMap[tagText] = Core(
        blocs: blocs,
        tag: tagText,
        //  views: this.views,
      );
    } else {
      if (EasyManager.debugMode) {
        print("EasyManager START AGAIN ($tagText)");
      }
      _injectMapHelper[tagText] = Core(
        blocs: blocs,
        tag: tagText,
        //  views: this.views,
      );
    }
  }
}

class _EasyManagerListState extends State<EasyManager> {
  @override
  void initState() {
    super.initState();
    if(widget.bloc != null){
    
     var unitBloc = Bloc((i) => widget.bloc);
     
     widget.blocs.add(unitBloc);
     EasyManager.addCoreInit(widget.blocs, widget.tagText);
    }
    
  }

  @override
  void dispose() {
    Core core = _injectMap[widget.tagText];
    core?.dispose();
    if (_injectMapHelper.containsKey(widget.tagText)) {
      _injectMap[widget.tagText] = _injectMapHelper[widget.tagText];
      _injectMapHelper.remove(widget.tagText);
      if (EasyManager.debugMode) {
        print(" --- DISPOSE BLOC ADDED ---- (${widget.tagText})");
      }
    } else {
      _injectMap.remove(widget.tagText);
      if (EasyManager.debugMode) {
        print(" --- DISPOSE BLOC PROVIDER---- (${widget.tagText})");
      }
    }

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return widget.view;
  }
}

class EasyManagerException implements Exception {
  String cause;
  EasyManagerException(this.cause);

  @override
  String toString() {
    return "EasyManager: $cause";
  }
}